#version 330

uniform vec3 Ambient;
uniform vec3 LightColor;
uniform vec3 LightDirection;
uniform vec3 HalfVector;
uniform float Shininess;
uniform float Strength;

uniform sampler2D Texture;

// the data to be sent to the fragment shader
in FragmentData {
    vec3 normal;
    vec4 color;
    vec2 texCoord;
} FDataIn;

out vec4 FragColor;

void main(void)
{
    // compute cosine of the directions, using dot products,
    // to see how much light would be reflected
    float diffuse = max(0.0, dot(FDataIn.normal, LightDirection));
    float specular = max(0.0, dot(FDataIn.normal, HalfVector));

    // surfaces facing away from the light (negative dot products)
    // won’t be lit by the directional light
    if (diffuse == 0.0)
       specular = 0.0;
    else
       specular = pow(specular, Shininess); // sharpen the highlight    

    vec3 scatteredLight = Ambient + LightColor * diffuse;
    vec3 reflectedLight = LightColor * specular * Strength;

    // don’t modulate the underlying color with reflected light,
    // only with scattered light
    vec3 lrgb = min(scatteredLight + reflectedLight, vec3(1.0));
    
    // weighted by the attrributed color
    vec3 rgb = min(FDataIn.color.rgb * scatteredLight + reflectedLight, vec3(1.0));

    // weighted by the texture
    vec3 texC = min(vec3(texture(Texture, FDataIn.texCoord))* scatteredLight + reflectedLight, vec3(1.0));

    //FragColor = FDataIn.color;     // Output: attributed color
    //FragColor = texture(Texture, FDataIn.texCoord);  //Output: attributed texture
    //FragColor = vec4(lrgb, 1.0);    // Output: computed lighting
    //FragColor = vec4(rgb, FDataIn.color.a); // Output: attributed color weighted by lighting
    //FragColor = vec4(texC, 1.0);; // Output: attributed texture weighted by lighting
    FragColor = vec4(texC*rgb, 1.0);; // Output: attributed texture weighted by color and lighting
}
